//  File:  NDNew.cpp

#include <iostream>
#include <iomanip>

using namespace std;

/*------------------------------------------------------------------*/
/*  This example shows how to set up a dynamic 2-D array of arrays. */
/*  Note that the arrays in the second dimension do NOT all have to */
/*  be of the same length...  If they're not, then you'll need to   */
/*  tell how long each one is -- because C++ has no way to know...  */
/*                                                                  */
/*  Last updated:  April 17, 2000                                   */
/*------------------------------------------------------------------*/

// declare the "integer pointer" type
typedef  int*  IntPtr;

void ArrayOfArrays(void)
{
	void Pass2Darray(IntPtr *p, const int M2, const int N3);

	cout << "Dynamic 2-D Array of Arrays" << endl << endl;

	int i, j, *q;
	int M2 = 2, N3 = 3;  // Note that M2 and N3 could be ANY values!

						 // declare p to be the BASE of the 2-D array, and
						 // set up the i-dimension (the vertical one)
	IntPtr *p = new IntPtr[M2];

	// set up the j-dimensions (the horizontal ones)
	for (i = 0; i < M2; i++)
		p[i] = new int[N3];

	// load array[i,j] with  10*i+j
	for (i = 0; i < M2; i++)
	{
		q = p[i];
		for (j = 0; j < N3; j++)
			q[j] = 10 * i + j;
	}

	// display the array in 2-D, as an array of arrays
	for (i = 0; i < M2; i++)
	{
		q = p[i];
		for (j = 0; j < N3; j++)
			cout << setw(4) << q[j];
		cout << endl;
	}
	cout << endl;

	// display the array again, as a 2-D array
	// (most of the time, THIS is what you'll really want to use)
	for (i = 0; i < M2; i++)
	{
		for (j = 0; j < N3; j++)
			cout << setw(4) << p[i][j];
		cout << endl;
	}
	cout << endl;

	// now, pass the array to a routine...
	Pass2Darray(p, M2, N3);

	// now, deallocate the array
	for (i = 0; i < M2; i++)
		delete[] p[i];
	delete[] p;
}

void Pass2Darray(IntPtr *p, const int M2, const int N3)
{
	cout << "Update, and then show what is inside p" << endl << endl;

	int  i, j;

	for (i = 0; i < M2; i++)
		for (j = 0; j < N3; j++)
			p[i][j] += 11;

	for (i = 0; i < M2; i++)
	{
		for (j = 0; j < N3; j++)
			cout << setw(4) << p[i][j];
		cout << endl;
	}
	cout << endl;
}

/*------------------------------------------------------------------*/
/*  This example shows how to set up a dynamic 3-D array.           */
/*  Note that the dynamic array truly acts like a "normal" 3-D array*/
/*------------------------------------------------------------------*/

void Dynamic3D(void)
{
	cout << "True Dynamic 3-D Array" << endl << endl;

	int i, j, k;

	// note how this is done...  new returns an object of type (int*)[4][5].
	// Note the parentheses around the *p -- needed due to how C++ parses.
	// We KNOW IN ADVANCE that the array will be 3 x 4 x 5,
	// but we want to allocate it dynamically rather than statically.
	// To allocate it statically we would have said:  int p[3][4][5];
	int(*p)[4][5] = new int[3][4][5];

	// load up array[i][j][k] with 100*i+10*j+k
	for (i = 0; i < 3; i++)
		for (j = 0; j < 4; j++)
			for (k = 0; k < 5; k++)
				p[i][j][k] = 100 * i + 10 * j + k;

	// display the contents of the array
	for (i = 0; i < 3; i++)
	{
		for (j = 0; j < 4; j++)
		{
			for (k = 0; k < 5; k++)
				cout << setw(5) << p[i][j][k];
			cout << endl;
		}
		cout << endl;
	}

	delete[] p;
}

/*------------------------------------------------------------------*/
/*    the main routine                                              */
/*------------------------------------------------------------------*/

int main(void)
{
	cout << "Show how to create and use dynamically allocated arrays"
		<< endl << endl;
	ArrayOfArrays();
	Dynamic3D();

	cout << "End Demo" << endl << endl;

	return 0;
}

/*------------------  ----- Output -----

Show how to create and use dynamically allocated arrays

Dynamic 2-D Array of Arrays

0   1   2
10  11  12

True Dynamic 3-D Array

0    1    2    3    4
10   11   12   13   14
20   21   22   23   24
30   31   32   33   34

100  101  102  103  104
110  111  112  113  114
120  121  122  123  124
130  131  132  133  134

200  201  202  203  204
210  211  212  213  214
220  221  222  223  224
230  231  232  233  234

*/
